// iqPolarize.cpp: dfinit le point d'entre pour l'application console.
//

#include "stdafx.h"
#ifdef linux
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include <errno.h>
#include <math.h>
#include <float.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <unistd.h>
#endif


double monRand(double x);
int polarize(int Rota, int hidd, int nbsections, double largarc, int attempts);
void exitHelp();


double pi = atan(1.0)*4.0;

double monRand(double x)
{
	return (rand() * x) / RAND_MAX;
}

int polarize(int Rota, int hidd, int nbsections, double largarc, int attempts)
{
	double lePas = largarc * pi / (180.0* nbsections),
		pasd = (hidd + hidd - Rota - Rota - largarc)*pi / 360.0,
		tp = cos(pasd),
		tq = sin(pasd);

	for (int i = 1; i < nbsections; i++, pasd += lePas)
	{
		tp *= cos(pasd);
		tq *= sin(pasd);
	}
	// power 2 once
	tp *= tp;
	tq *= tq;

	// some
	if (tp + tq == 0.0)
		return 0;

	// ignore the (1-tp-tq) hole
	double randoo = monRand(1.0), zozo = (1.0 - pow(1.0 - tp - tq, attempts)) / (tp + tq);

	if (randoo <= tp*zozo)
	{
		return 1;
	}
	else if (randoo >= 1.0 - tq*zozo)
	{
		return -1;
	}
	else
	{
		return 0;
	}

}


void exitHelp()
{
	printf("outputs after-polarizer data for the numerical EPR experiment :\n");
	printf("# qPolarize width_around number_of_sections number_of_attempts hidden_variable_file rotation_file output_file\n");
	printf("Both arguments are mandatory\n");
	printf("0 < number_of_sections < 50\n");
	exit(1);
}


int main(int argc, char *argv[])
{
	int	nbSections, Attempts;
	double	LargArc;
	char	fichierINVAR[512],
		fichierINROT[512],
		fichierOUT[512];

	FILE  *fcbINVAR, *fcbINROT, *fcbOUT;

	// extract arguments

	if (argc != 7)
	{
		exitHelp();
	}

	LargArc = atof(argv[1]);
	if (LargArc < 0.0 || LargArc > 32.0)
	{
		printf("The width around the target is out of range : 0 to 32\n");
		printf("It has been changed to 16\n");
		LargArc = 16;
	}

	nbSections = atoi(argv[2]);
	if (nbSections < 1 || nbSections > 1000)
	{
		printf("The number_of_sections is out of range : 1 to 1000\n");
		printf("It has been changed to 11\n");
		nbSections = 11;
	}

	Attempts = atoi(argv[3]);
	if (Attempts < 1 || Attempts > 2000)
	{
		printf("The number_of_attempts is out of range : 1 to 2000\n");
		printf("It has been changed to 200\n");
		Attempts = 200;
	}

	strcpy(fichierINVAR, argv[4]);
	if (!*fichierINVAR)
	{
		exitHelp();
	}

	strcpy(fichierINROT, argv[5]);
	if (!*fichierINROT)
	{
		exitHelp();
	}

	strcpy(fichierOUT, argv[6]);
	if (!*fichierOUT)
	{
		exitHelp();
	}

	// ################################################################################################

	// opening files
	//  *fcbINVAR, *fcbINROT, *fcbOUT;

	if (!(fcbOUT = fopen(fichierOUT, "w")))
	{
		printf("unable to create the result file %s\n", fichierOUT);
		exit(1);
	}

	if (!(fcbINVAR = fopen(fichierINVAR, "r")))
	{
		printf("unable to read the variables file %s\n", fichierINVAR);
		fclose(fcbOUT);
		exit(1);
	}

	if (!(fcbINROT = fopen(fichierINROT, "r")))
	{
		printf("unable to read the rotations file %s\n", fichierINROT);
		fclose(fcbINVAR);
		fclose(fcbOUT);
		exit(1);
	}

	// seed the randomizer
	srand((unsigned int)time(NULL)); //  seed48()  and  lcong48()

	// now arguments are ready and files are open

	char bufVAR[512], bufROT[512];
	int Hid, Rot, nblignes = 0;

	while (fgets(bufVAR, 512, fcbINVAR) && fgets(bufROT, 512, fcbINROT))
	{
		nblignes++;
		Rot = atoi(bufROT);
		Hid = atoi(bufVAR);
		// got 1 variable and 1 rotation

		// compute the result and output
		fprintf(fcbOUT, "%d %d\n", Rot, polarize(Rot, Hid, nbSections, LargArc, Attempts));

	}
	fclose(fcbINVAR);
	fclose(fcbINROT);
	fclose(fcbOUT);

	printf("%d elements went through the polarizer.\n", nblignes);

	exit(0);

}


